/*
* Copyright (C) 2011 Freescale Semiconductor, Inc. All rights reserved.
*
*/

/*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Library General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Library General Public License for more details.
*
* You should have received a copy of the GNU Library General Public
* License along with this library; if not, write to the
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,
* Boston, MA 02111-1307, USA.
*/

/*
* Module Name:    gstsutils.c
*
* Description:    simple utils for gst plugins
*
* Portability:    This code is written for Linux OS and Gstreamer
*/

/*
* Changelog:
*
*/

#include "gstsutils.h"
static gboolean
g_string_to_boolean (gchar * str)
{
  if ((str) && ((strcmp (str, "true") == 0) || (strcmp (str, "TRUE") == 0))) {
    return TRUE;
  }
  return FALSE;
}

void
gstsutils_options_install_properties_by_options (GstsutilsOptionEntry * table,
    GObjectClass * oclass)
{
  GstsutilsOptionEntry *p = table;
  while (p->id >= 0) {
    switch (p->gtype) {
      case (G_TYPE_BOOLEAN):
        g_object_class_install_property (oclass, p->id,
            g_param_spec_boolean (p->name, p->nickname,
                p->desc, g_string_to_boolean (p->def), G_PARAM_READWRITE));
        break;
      case (G_TYPE_INT64):
        g_object_class_install_property (oclass, p->id,
            g_param_spec_int64 (p->name, p->nickname,
                p->desc,
                g_ascii_strtoll (p->min, NULL, 0), g_ascii_strtoll (p->max,
                    NULL, 0), g_ascii_strtoll (p->def, NULL, 0),
                G_PARAM_READWRITE));
        break;
      case (G_TYPE_INT):
        g_object_class_install_property (oclass, p->id,
            g_param_spec_int (p->name, p->nickname,
                p->desc,
                (gint) g_ascii_strtoll (p->min, NULL, 0),
                (gint) g_ascii_strtoll (p->max, NULL, 0),
                (gint) g_ascii_strtoll (p->def, NULL, 0), G_PARAM_READWRITE));
        break;
      case (G_TYPE_DOUBLE):
        g_object_class_install_property (oclass, p->id,
            g_param_spec_double (p->name, p->nickname,
                p->desc,
                g_strtod (p->min, NULL),
                g_strtod (p->max, NULL),
                g_strtod (p->def, NULL), G_PARAM_READWRITE));
        break;
      default:
        break;
    };
    p++;
  };
}

static GstsutilsOptionEntry *
gstsutils_options_search_option_by_id (GstsutilsOptionEntry * table, gint id)
{
  if (table) {
    while (table->id != -1) {
      if (table->id == id) {
        return table;
      }
      table++;
    }
  }
  return NULL;
}

gboolean
gstsutils_options_get_option (GstsutilsOptionEntry * table, gchar * option,
    guint id, GValue * value)
{
  gboolean ret = TRUE;
  if ((value) && (option)) {
    GstsutilsOptionEntry *p = gstsutils_options_search_option_by_id (table, id);
    if (p) {
      switch (p->gtype) {
        case (G_TYPE_BOOLEAN):
          g_value_set_boolean (value, *(gboolean *) (option + p->offset));
          break;
        case (G_TYPE_INT64):
          g_value_set_int64 (value, *(gint64 *) (option + p->offset));
          break;
        case (G_TYPE_INT):
          g_value_set_int (value, *(gint *) (option + p->offset));
          break;
        case (G_TYPE_DOUBLE):
          g_value_set_double (value, *(gdouble *) (option + p->offset));
          break;
        default:
          ret = FALSE;
          break;
      };
    }
  } else {
    ret = FALSE;
  }
  return ret;
}

gboolean
gstsutils_options_set_option (GstsutilsOptionEntry * table, gchar * option,
    guint id, GValue * value)
{
  gboolean ret = TRUE;
  if ((value) && (option)) {
    GstsutilsOptionEntry *p = gstsutils_options_search_option_by_id (table, id);
    if (p) {
      switch (p->gtype) {
        case (G_TYPE_BOOLEAN):
          *(gboolean *) (option + p->offset) = g_value_get_boolean (value);
          break;
        case (G_TYPE_INT64):
          *(gint64 *) (option + p->offset) = g_value_get_int64 (value);
          break;
        case (G_TYPE_INT):
          *(gint *) (option + p->offset) = g_value_get_int (value);
          break;
        case (G_TYPE_DOUBLE):
          *(gdouble *) (option + p->offset) = g_value_get_double (value);
          break;
        default:
          ret = FALSE;
          break;
      };
    }
  } else {
    ret = FALSE;
  }
  return ret;
}

void
gstsutils_options_load_default (GstsutilsOptionEntry * table, gchar * option)
{
  if ((table) && (option)) {
    GstsutilsOptionEntry *p = table;
    while (p->id != -1) {
      switch (p->gtype) {
        case (G_TYPE_BOOLEAN):
          *(gboolean *) (option + p->offset) = g_string_to_boolean (p->def);
          break;
        case (G_TYPE_INT64):
          *(gint64 *) (option + p->offset) = g_ascii_strtoll (p->def, NULL, 0);
          break;
        case (G_TYPE_INT):
          *(gint *) (option + p->offset) =
              (gint) g_ascii_strtoll (p->def, NULL, 0);
          break;
        case (G_TYPE_DOUBLE):
          *(gdouble *) (option + p->offset) = g_strtod (p->def, NULL);
          break;
        default:
          break;
      };
      p++;
    }
  }
}

gboolean
gstsutils_options_load_from_keyfile (GstsutilsOptionEntry * table,
    gchar * option, gchar * filename, gchar * group)
{
  GKeyFile *keyfile = NULL;
  gboolean ret = FALSE;

  if (filename == NULL)
    goto bail;

  if ((option) && (keyfile = g_key_file_new ())
      && (g_key_file_load_from_file (keyfile, filename, G_KEY_FILE_NONE, NULL))) {
    GstsutilsOptionEntry *p = table;

    while (p->id != -1) {
      if (g_key_file_has_key (keyfile, group, p->name, NULL)) {
        switch (p->gtype) {
          case (G_TYPE_BOOLEAN):
            *(gboolean *) (option + p->offset) =
                g_key_file_get_boolean (keyfile, group, p->name, NULL);
            break;
          case (G_TYPE_INT64):
          {
#if 0
            gint64 value = g_key_file_get_int64 (keyfile, group, p->name, NULL);
#else
            gint64 value = 0;
            gchar *svalue =
                g_key_file_get_value (keyfile, group, p->name, NULL);

            if (svalue) {
              value = g_ascii_strtoll (svalue, NULL, 0);
              g_free (svalue);
#endif
              if ((value >= g_ascii_strtoll (p->min, NULL, 0))
                  && (value <= g_ascii_strtoll (p->max, NULL, 0))) {
                *(gint64 *) (option + p->offset) = value;
              }
#if 0
#else
            }
#endif
            break;
          }
          case (G_TYPE_INT):
          {
            gint value = g_key_file_get_integer (keyfile, group, p->name, NULL);
            if ((value >= (gint) g_ascii_strtoll (p->min, NULL, 0))
                && (value <= (gint) g_ascii_strtoll (p->max, NULL, 0))) {
              *(gint *) (option + p->offset) = value;
            }
            break;
          }
          case (G_TYPE_DOUBLE):
          {
            gdouble value =
                g_key_file_get_double (keyfile, group, p->name, NULL);
            if ((value >= g_strtod (p->min, NULL))
                && (value <= g_strtod (p->max, NULL))) {
              *(gdouble *) (option + p->offset) = value;
            }
            break;
          }
          default:
            break;
        };
      }
      p++;
    };
    ret = TRUE;
  }

bail:
  if (keyfile) {
    g_key_file_free (keyfile);
  }
  return ret;
}
